@@ -10,24 +10,6 @@
 ;;;; files for more information.
 
 (in-package "SB!VM")
-
-;;;; Interfaces to IR2 conversion:
-
-;;; Return a wired TN describing the N'th full call argument passing
-;;; location.
-(defun standard-arg-location (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-wired-tn *backend-t-primitive-type* register-arg-scn
-                     (elt *register-arg-offsets* n))
-      (make-wired-tn *backend-t-primitive-type* control-stack-arg-scn n)))
-
-(defun standard-arg-location-sc (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-sc-offset register-arg-scn
-                      (nth n *register-arg-offsets*))
-      (make-sc-offset control-stack-arg-scn n)))
 
 (defconstant arg-count-sc (make-sc-offset immediate-arg-scn nargs-offset))
 (defconstant closure-sc (make-sc-offset descriptor-reg-sc-number lexenv-offset))
@@ -84,36 +66,6 @@
 ;;; passed when we are using non-standard conventions.
 (defun make-arg-count-location ()
   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nargs-offset))
-
-
-;;; Make a TN to hold the number-stack frame pointer.  This is
-;;; allocated once per component, and is component-live.
-(defun make-nfp-tn ()
-  (component-live-tn
-   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nfp-offset)))
-
-(defun make-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-(defun make-number-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-;;; Return a list of TNs that can be used to represent an unknown-values
-;;; continuation within a function.
-(defun make-unknown-values-locations ()
-  (list (make-stack-pointer-tn)
-        (make-normal-tn *fixnum-primitive-type*)))
-
-;;; This function is called by the ENTRY-ANALYZE phase, allowing
-;;; VM-dependent initialization of the IR2-COMPONENT structure.  We push
-;;; placeholder entries in the Constants to leave room for additional
-;;; noise in the code object header.
-(defun select-component-format (component)
-  (declare (type component component))
-  (dotimes (i code-constants-offset)
-    (vector-push-extend nil
-                        (ir2-component-constants (component-info component))))
-  (values))
 
 ;;;; Frame hackery:
 
@@ -346,15 +298,14 @@
 ;;; thing this handles is allocation of the result temporaries.
 (define-vop (unknown-values-receiver)
   (:results
-   (start :scs (any-reg))
-   (count :scs (any-reg)))
-  (:temporary (:sc descriptor-reg :offset ocfp-offset
-                   :from :eval :to (:result 0))
+   (start :scs (any-reg) :from (:save 0))
+   (count :scs (any-reg) :from (:save 1)))
+  (:temporary (:sc any-reg :offset ocfp-offset
+               :from :eval :to :save)
               values-start)
   (:temporary (:sc any-reg :offset nargs-offset
-               :from :eval :to (:result 1))
-              nvals)
-  (:temporary (:scs (non-descriptor-reg)) temp))
+               :from :eval :to :save)
+              nvals))
 
 ;;; This hook in the codegen pass lets us insert code before fall-thru entry
 ;;; points, local-call entry points, and tail-call entry points.  The default
@@ -506,7 +457,7 @@
   (:temporary (:scs (any-reg) :from (:argument 1)) count)
   (:temporary (:scs (descriptor-reg) :from :eval) temp)
   (:temporary (:scs (any-reg) :from :eval) dst)
-  (:temporary (:sc non-descriptor-reg) alloc-temp pa-flag)
+  (:temporary (:sc non-descriptor-reg) pa-flag)
   (:temporary (:scs (interior-reg)) lip)
   (:results (result :scs (descriptor-reg)))
   (:translate %listify-rest-args)
@@ -531,7 +482,7 @@
         (allocation dst size list-pointer-lowtag
                     :flag-tn pa-flag
                     :stack-allocate-p dx-p
-                    :lip lip :temp alloc-temp))
+                    :lip lip))
       (move result dst)
 
       (inst b ENTER)
@@ -608,31 +559,6 @@
              ((plusp min)
               (inst cmp nargs (load-immediate min))
               (inst b :lo err-lab)))))))
-
-;;; Signal various errors.
-(macrolet ((frob (name error translate &rest args)
-             `(define-vop (,name)
-                ,@(when translate
-                    `((:policy :fast-safe)
-                      (:translate ,translate)))
-                (:args ,@(mapcar #'(lambda (arg)
-                                     `(,arg :scs (any-reg descriptor-reg)))
-                                 args))
-                (:vop-var vop)
-                (:save-p :compute-only)
-                (:generator 1000
-                  (error-call vop ',error ,@args)))))
-  (frob arg-count-error invalid-arg-count-error
-    sb!c::%arg-count-error nargs fname)
-  (frob type-check-error object-not-type-error sb!c::%type-check-error
-    object type)
-  (frob layout-invalid-error layout-invalid-error sb!c::%layout-invalid-error
-    object layout)
-  (frob odd-key-args-error odd-key-args-error
-        sb!c::%odd-key-args-error)
-  (frob unknown-key-arg-error unknown-key-arg-error
-        sb!c::%unknown-key-arg-error key)
-  (frob nil-fun-returned-error nil-fun-returned-error nil fun))
 
 ;;;; Local call with unknown values convention return:
 
@@ -879,7 +805,7 @@
                       :to :eval)
                  ,(if named 'name-pass 'lexenv))
 
-     (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
+     (:temporary (:scs (descriptor-reg) :to :eval)
                function)
      (:temporary (:sc any-reg :offset nargs-offset :to :eval)
                  nargs-pass)
@@ -888,12 +814,12 @@
          (mapcar #'(lambda (name offset)
                      `(:temporary (:sc descriptor-reg
                                    :offset ,offset
-                                   :to :eval)
+                                   :to :result)
                          ,name))
                  *register-arg-names* *register-arg-offsets*))
      ,@(when (eq return :fixed)
          '((:temporary (:scs (descriptor-reg) :from :eval) move-temp)
-           (:temporary (:sc non-descriptor-reg :from :eval :offset ocfp-offset) ocfp-temp)))
+           (:temporary (:sc any-reg :from :eval :offset ocfp-offset) ocfp-temp)))
 
      ,@(unless (eq return :tail)
          '((:temporary (:scs (non-descriptor-reg)) temp)
@@ -967,7 +893,10 @@
                   ;; Conditionally insert a conditional trap:
                   (when step-instrumenting
                     (assemble ()
+                      #!-sb-thread
                       (load-symbol-value tmp-tn sb!impl::*stepping*)
+                      #!+sb-thread
+                      (loadw tmp-tn thread-tn thread-stepping-slot)
                       (inst cbz tmp-tn step-done-label)
                       ;; CONTEXT-PC will be pointing here when the
                       ;; interrupt is handled, not after the
@@ -1074,8 +1003,7 @@
         (inst add cur-nfp cur-nfp (add-sub-immediate
                                    (bytes-needed-for-non-descriptor-stack-frame)))
         (inst mov-sp nsp-tn cur-nfp)))
-    (load-fixup tmp-tn (make-fixup 'tail-call-variable :assembly-routine)
-                lip)
+    (load-inline-constant tmp-tn '(:fixup tail-call-variable :assembly-routine) lip)
     (inst br tmp-tn)))
 
 ;;;; Unknown values return:
@@ -1199,18 +1127,20 @@
     (move old-fp old-fp-arg)
     (move vals vals-arg)
     (move nvals nvals-arg)
-    (load-fixup tmp-tn (make-fixup 'return-multiple :assembly-routine) lip)
+    (load-inline-constant tmp-tn '(:fixup return-multiple :assembly-routine) lip)
     (inst br tmp-tn)))
 
 ;;; Single-stepping
 
 (define-vop (step-instrument-before-vop)
-  (:temporary (:scs (descriptor-reg)) stepping)
   (:policy :fast-safe)
   (:vop-var vop)
   (:generator 3
-    (load-symbol-value stepping sb!impl::*stepping*)
-    (inst cbz stepping DONE)
+    #!-sb-thread
+    (load-symbol-value tmp-tn sb!impl::*stepping*)
+    #!+sb-thread
+    (loadw tmp-tn thread-tn thread-stepping-slot)
+    (inst cbz tmp-tn DONE)
     ;; CONTEXT-PC will be pointing here when the interrupt is handled,
     ;; not after the BREAK.
     (note-this-location vop :step-before-vop)
