@@ -18,14 +18,14 @@
 (defknown coerce (t type-specifier) t
     ;; Note:
     ;; This is not FLUSHABLE because it's defined to signal errors.
-    (movable explicit-check)
+    (movable)
   ;; :DERIVE-TYPE RESULT-TYPE-SPEC-NTH-ARG 2 ? Nope... (COERCE 1 'COMPLEX)
   ;; returns REAL/INTEGER, not COMPLEX.
   )
 ;; These each check their input sequence for type-correctness,
 ;; but not the output type specifier, because MAKE-SEQUENCE will do that.
-(defknown list-to-vector* (list type-specifier) vector (explicit-check))
-(defknown vector-to-vector* (vector type-specifier) vector (explicit-check))
+(defknown list-to-vector* (list type-specifier) vector ())
+(defknown vector-to-vector* (vector type-specifier) vector ())
 
 ;; FIXME: Is this really FOLDABLE? A counterexample seems to be:
 ;;  (LET ((S :S)) (VALUES (TYPE-OF S) (UNINTERN S 'KEYWORD) (TYPE-OF S) S))
@@ -34,12 +34,12 @@
 ;; CLASS can be returned only when you've got an object whose class-name
 ;; does not properly name its class.
 (defknown type-of (t) (or list symbol class)
-  (foldable flushable explicit-check))
+  (foldable flushable))
 
 ;;; These can be affected by type definitions, so they're not FOLDABLE.
 (defknown (sb!xc:upgraded-complex-part-type sb!xc:upgraded-array-element-type)
     (type-specifier &optional lexenv-designator) (or list symbol)
-    (unsafely-flushable explicit-check))
+    (unsafely-flushable))
 
 ;;;; from the "Predicates" chapter:
 
@@ -67,7 +67,7 @@
    ;;
    ;; (UPGRADED-ARRAY-ELEMENT-TYPE and UPGRADED-COMPLEX-PART-TYPE have
    ;; behavior like SUBTYPEP in this respect, not like TYPEP.)
-   (foldable explicit-check))
+   (foldable))
 (defknown subtypep (type-specifier type-specifier &optional lexenv-designator)
   (values boolean boolean)
   ;; This is not FOLDABLE because its value is affected by type
@@ -75,7 +75,7 @@
   ;;
   ;; FIXME: Is it OK to fold this when the types have already been
   ;; defined? Does the code inherited from CMU CL already do this?
-  (unsafely-flushable explicit-check))
+  (unsafely-flushable))
 
 (defknown (null symbolp atom consp listp numberp integerp rationalp floatp
                 complexp characterp stringp bit-vector-p vectorp
@@ -109,7 +109,7 @@
 ;;; unbound.
 (defknown (symbol-value) (symbol) t ())
 (defknown about-to-modify-symbol-value (symbol t &optional t t) null
-  (explicit-check))
+  ())
 ;;; From CLHS, "If the symbol is globally defined as a macro or a
 ;;; special operator, an object of implementation-dependent nature and
 ;;; identity is returned. If the symbol is not globally defined as
@@ -119,21 +119,19 @@
 (defknown (symbol-function) (symbol) function ())
 
 (defknown boundp (symbol) boolean (flushable))
-(defknown fboundp ((or symbol cons)) boolean (unsafely-flushable explicit-check))
+(defknown fboundp ((or symbol cons)) boolean (unsafely-flushable))
 (defknown special-operator-p (symbol) t
   ;; The set of special operators never changes.
   (movable foldable flushable))
 (defknown set (symbol t) t ()
   :derive-type #'result-type-last-arg)
-(defknown fdefinition ((or symbol cons)) function (explicit-check))
-(defknown %set-fdefinition ((or symbol cons) function) function
-  (explicit-check)
+(defknown fdefinition ((or symbol cons)) function ())
+(defknown %set-fdefinition ((or symbol cons) function) function ()
   :derive-type #'result-type-last-arg)
-(defknown makunbound (symbol) symbol
-  ()
+(defknown makunbound (symbol) symbol ()
   :derive-type #'result-type-first-arg)
 (defknown fmakunbound ((or symbol cons)) (or symbol cons)
-  (explicit-check)
+  ()
   :derive-type #'result-type-first-arg)
 (defknown apply (callable t &rest t) *) ; ### Last arg must be List...
 (defknown funcall (callable &rest t) *)
@@ -234,75 +232,75 @@
 
 ;;;; from the "Numbers" chapter:
 
-(defknown zerop (number) boolean (movable foldable flushable explicit-check))
+(defknown zerop (number) boolean (movable foldable flushable))
 (defknown (plusp minusp) (real) boolean
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown (oddp evenp) (integer) boolean
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown (=) (number &rest number) boolean
-  (movable foldable flushable explicit-check commutative))
+  (movable foldable flushable commutative))
 (defknown (/=) (number &rest number) boolean
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown (< > <= >=) (real &rest real) boolean
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown (max min) (real &rest real) real
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown (+ *) (&rest number) number
-  (movable foldable flushable explicit-check commutative))
+  (movable foldable flushable commutative))
 (defknown - (number &rest number) number
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown / (number &rest number) number
-  (movable foldable unsafely-flushable explicit-check))
+  (movable foldable unsafely-flushable))
 (defknown (1+ 1-) (number) number
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown conjugate (number) number
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown gcd (&rest integer) unsigned-byte
-  (movable foldable flushable explicit-check)
+  (movable foldable flushable)
   #|:derive-type 'boolean-result-type|#)
 (defknown lcm (&rest integer) unsigned-byte
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 #+sb-xc-host ; (See CROSS-FLOAT-INFINITY-KLUDGE.)
 (defknown exp (number) irrational
-  (movable foldable flushable explicit-check recursive)
+  (movable foldable flushable recursive)
   :derive-type #'result-type-float-contagion)
 
 #-sb-xc-host ; (See CROSS-FLOAT-INFINITY-KLUDGE.)
 (defknown exp (number) irrational
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 
 (defknown expt (number number) number
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 (defknown log (number &optional real) irrational
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 (defknown sqrt (number) irrational
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown isqrt (unsigned-byte) unsigned-byte
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 
 (defknown (abs phase signum) (number) number
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown cis (real) (complex float)
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 #+sb-xc-host ; (See CROSS-FLOAT-INFINITY-KLUDGE.)
 (progn
 (defknown (sin cos) (number)
   (or (float -1.0 1.0) (complex float))
-  (movable foldable flushable explicit-check recursive)
+  (movable foldable flushable recursive)
   :derive-type #'result-type-float-contagion)
 
 (defknown atan
   (number &optional real) irrational
-  (movable foldable unsafely-flushable explicit-check recursive)
+  (movable foldable unsafely-flushable recursive)
   :derive-type #'result-type-float-contagion)
 
 (defknown (tan sinh cosh tanh asinh)
-  (number) irrational (movable foldable flushable explicit-check recursive)
+  (number) irrational (movable foldable flushable recursive)
   :derive-type #'result-type-float-contagion)
 ) ; PROGN
 
@@ -310,90 +308,90 @@
 (progn
 (defknown (sin cos) (number)
   (or (float -1.0 1.0) (complex float))
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 
 (defknown atan
   (number &optional real) irrational
-  (movable foldable unsafely-flushable explicit-check recursive))
+  (movable foldable unsafely-flushable recursive))
 
 (defknown (tan sinh cosh tanh asinh)
-  (number) irrational (movable foldable flushable explicit-check recursive))
+  (number) irrational (movable foldable flushable recursive))
 ) ; PROGN
 
 (defknown (asin acos acosh atanh)
   (number) irrational
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 
 (defknown float (real &optional float) float
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown (rational) (real) rational
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown (rationalize) (real) rational
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 
 (defknown (numerator denominator) (rational) integer
   (movable foldable flushable))
 
 (defknown (floor ceiling round)
   (real &optional real) (values integer real)
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown truncate
   (real &optional real) (values integer real)
-  (movable foldable flushable explicit-check recursive))
+  (movable foldable flushable recursive))
 
 (defknown %multiply-high (word word) word
     (movable foldable flushable))
 
 (defknown (mod rem) (real real) real
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown (ffloor fceiling fround ftruncate)
   (real &optional real) (values float real)
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown decode-float (float) (values float float-exponent float)
-  (movable foldable unsafely-flushable explicit-check))
+  (movable foldable unsafely-flushable))
 (defknown scale-float (float integer) float
-  (movable foldable unsafely-flushable explicit-check))
+  (movable foldable unsafely-flushable))
 (defknown float-radix (float) float-radix
   (movable foldable unsafely-flushable))
 (defknown float-sign (float &optional float) float
-  (movable foldable unsafely-flushable explicit-check))
+  (movable foldable unsafely-flushable))
 (defknown (float-digits float-precision) (float) float-digits
-  (movable foldable unsafely-flushable explicit-check))
+  (movable foldable unsafely-flushable))
 (defknown integer-decode-float (float)
     (values integer float-int-exponent (member -1 1))
-    (movable foldable unsafely-flushable explicit-check))
+    (movable foldable unsafely-flushable))
 
 (defknown complex (real &optional real) number
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown (realpart imagpart) (number) real (movable foldable flushable))
 
 (defknown (logior logxor logand logeqv) (&rest integer) integer
-  (movable foldable flushable explicit-check commutative))
+  (movable foldable flushable commutative))
 
 (defknown (lognand lognor logandc1 logandc2 logorc1 logorc2)
           (integer integer) integer
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown boole (boole-code integer integer) integer
   (movable foldable flushable))
 
-(defknown lognot (integer) integer (movable foldable flushable explicit-check))
+(defknown lognot (integer) integer (movable foldable flushable))
 (defknown logtest (integer integer) boolean (movable foldable flushable commutative))
 (defknown logbitp (unsigned-byte integer) boolean (movable foldable flushable))
 (defknown ash (integer integer) integer
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 #!+ash-right-vops
 (defknown %ash/right ((or word sb!vm:signed-word) (mod #.sb!vm:n-word-bits))
   (or word sb!vm:signed-word)
   (movable foldable flushable always-translatable))
 (defknown (logcount integer-length) (integer) bit-index
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 ;;; FIXME: According to the ANSI spec, it's legal to use any
 ;;; nonnegative indices for BYTE arguments, not just BIT-INDEX. It's
 ;;; hard to come up with useful ways to do this, but it is possible to
@@ -414,16 +412,14 @@
   (movable foldable flushable))
 (defknown random ((or (float (0.0)) (integer 1)) &optional random-state)
   (or (float 0.0) (integer 0))
-  (explicit-check))
+  ())
 (defknown make-random-state (&optional (or random-state (member nil t)))
   random-state (flushable))
-;; The arg type specifier is for documentation only- EXPLICIT-CHECK prevails.
-;; It feels almost wrong to state it at all.
 (defknown seed-random-state (&optional ; SBCL extension
                              (or (member nil t) random-state unsigned-byte
                                  (simple-array (unsigned-byte 8) (*))
                                  (simple-array (unsigned-byte 32) (*))))
-  random-state (flushable explicit-check))
+  random-state (flushable))
 
 (defknown random-state-p (t) boolean (movable foldable flushable))
 
@@ -449,7 +445,7 @@
 (defknown (char/= char< char> char<= char>= char-not-equal)
   (character &rest character) boolean (movable foldable flushable))
 (defknown (char-equal char-lessp char-greaterp char-not-greaterp char-not-lessp)
-  (character &rest character) boolean (movable foldable flushable explicit-check))
+  (character &rest character) boolean (movable foldable flushable))
 
 (defknown (two-arg-char-equal)
     (character character) boolean (movable foldable flushable commutative))
@@ -463,7 +459,7 @@
 
 (defknown char-equal-constant (character character character)
   boolean
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 
 (defknown character (t) character (movable foldable unsafely-flushable))
 (defknown char-code (character) char-code (movable foldable flushable))
@@ -497,12 +493,12 @@
 (defknown copy-seq (sequence) consed-sequence (flushable)
   :derive-type (sequence-result-nth-arg 1))
 
-(defknown length (sequence) index (explicit-check foldable flushable dx-safe))
+(defknown length (sequence) index (foldable flushable dx-safe))
 
-(defknown reverse (sequence) consed-sequence (explicit-check flushable)
+(defknown reverse (sequence) consed-sequence (flushable)
   :derive-type (sequence-result-nth-arg 1))
 
-(defknown nreverse (sequence) sequence (explicit-check important-result)
+(defknown nreverse (sequence) sequence (important-result)
   :derive-type #'result-type-first-arg
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 
@@ -510,30 +506,26 @@
                                         &key
                                         (:initial-element t))
   consed-sequence
-  (movable explicit-check)
+  (movable)
   :derive-type (creation-result-type-specifier-nth-arg 1))
 
-(defknown concatenate (type-specifier &rest sequence) consed-sequence
-  (explicit-check)
+(defknown concatenate (type-specifier &rest sequence) consed-sequence ()
   :derive-type (creation-result-type-specifier-nth-arg 1))
 
 (defknown %concatenate-to-string (&rest sequence) simple-string
-  (explicit-check flushable))
+  (flushable))
 (defknown %concatenate-to-base-string (&rest sequence) simple-base-string
-  (explicit-check flushable))
+  (flushable))
 
 (defknown map (type-specifier callable sequence &rest sequence)
-  consed-sequence (call explicit-check)
+  consed-sequence (call)
 ; :DERIVE-TYPE 'TYPE-SPEC-ARG1 ? Nope... (MAP NIL ...) returns NULL, not NIL.
   )
-(defknown %map (type-specifier callable &rest sequence)
-  consed-sequence (call explicit-check))
-(defknown %map-for-effect-arity-1 (callable sequence) null
-  (call explicit-check))
-(defknown %map-to-list-arity-1 (callable sequence) list
-  (flushable call explicit-check))
+(defknown %map (type-specifier callable &rest sequence) consed-sequence (call))
+(defknown %map-for-effect-arity-1 (callable sequence) null (call))
+(defknown %map-to-list-arity-1 (callable sequence) list (flushable call))
 (defknown %map-to-simple-vector-arity-1 (callable sequence) simple-vector
-  (flushable call explicit-check))
+  (flushable call))
 
 (defknown map-into (sequence callable &rest sequence)
   sequence
@@ -548,7 +540,7 @@
                           :sb!impl))
     (simple-array index index function list)
     index
-    (call explicit-check))
+    (call))
 
 ;;; returns the result from the predicate...
 (defknown some (callable sequence &rest sequence) t
@@ -709,7 +701,7 @@
   :derive-type (sequence-result-nth-arg 1)
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 (defknown sb!impl::stable-sort-list (list function function) list
-  (call important-result explicit-check)
+  (call important-result)
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 (defknown sb!impl::sort-vector (vector index index function (or function null))
   * ; SORT-VECTOR works through side-effect
@@ -719,19 +711,19 @@
 (defknown sb!impl::stable-sort-vector
   (vector function (or function null))
   vector
-  (call explicit-check)
+  (call)
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 
 (defknown sb!impl::stable-sort-simple-vector
   (simple-vector function (or function null))
   simple-vector
-  (call explicit-check)
+  (call)
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 
 (defknown merge (type-specifier sequence sequence callable
                                 &key (:key callable))
   sequence
-  (call important-result explicit-check)
+  (call important-result)
   :derive-type (creation-result-type-specifier-nth-arg 1)
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 2 3))
 
@@ -927,12 +919,10 @@
                       (:initial-element t)
                       (:initial-contents t)
                       (:adjustable t)
-                      ;; the type constraint doesn't do anything
-                      ;; on account of EXPLICIT-CHECK. it's documentation.
                       (:fill-pointer (or index boolean))
                       (:displaced-to (or array null))
                       (:displaced-index-offset index))
-  array (flushable explicit-check))
+  array (flushable))
 
 (defknown %make-array ((or index list)
                        (unsigned-byte #.sb!vm:n-widetag-bits)
@@ -947,9 +937,7 @@
                        (:displaced-index-offset index))
     array (flushable))
 
-(defknown sb!impl::fill-data-vector (vector list sequence)
-  t
-  (explicit-check))
+(defknown sb!impl::fill-data-vector (vector list sequence) t ())
 
 (defknown vector (&rest t) simple-vector (flushable))
 
@@ -994,17 +982,15 @@
 (defknown array-has-fill-pointer-p (array) boolean
   (movable foldable flushable))
 (defknown fill-pointer (complex-vector) index
-    (unsafely-flushable explicit-check))
+    (unsafely-flushable))
 (defknown sb!impl::fill-pointer-error (t &optional t) nil)
 
-(defknown vector-push (t complex-vector) (or index null)
-    (explicit-check)
+(defknown vector-push (t complex-vector) (or index null) ()
   :destroyed-constant-args (nth-constant-args 2))
 (defknown vector-push-extend (t complex-vector &optional (and index (integer 1))) index
-    (explicit-check)
+    ()
   :destroyed-constant-args (nth-constant-args 2))
-(defknown vector-pop (complex-vector) t
-    (explicit-check)
+(defknown vector-pop (complex-vector) t ()
   :destroyed-constant-args (nth-constant-args 1))
 
 ;;; FIXME: complicated :DESTROYED-CONSTANT-ARGS
@@ -1040,7 +1026,7 @@
 
 (defknown make-string (index &key (:element-type type-specifier)
                        (:initial-element character))
-  simple-string (flushable explicit-check))
+  simple-string (flushable))
 
 (defknown (string-trim string-left-trim string-right-trim)
   (sequence string-designator) string (flushable))
@@ -1054,8 +1040,7 @@
   string ()
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 
-(defknown string (string-designator) string
-  (flushable explicit-check))
+(defknown string (string-designator) string (flushable))
 
 ;;;; internal non-keyword versions of string predicates:
 
@@ -1087,7 +1072,7 @@
   (flushable))
 (defknown make-string-output-stream
     (&key (:element-type type-specifier)) sb!impl::string-output-stream
-  (flushable explicit-check))
+  (flushable))
 (defknown get-output-stream-string (stream) simple-string ())
 (defknown streamp (t) boolean (movable foldable flushable))
 (defknown stream-element-type (stream) type-specifier ; can it return a CLASS?
@@ -1164,34 +1149,30 @@
   (type-specifier (or null callable)
    &optional real sb!pretty:pprint-dispatch-table)
   null
-  (explicit-check))
+  ())
 
 ;;; may return any type due to eof-value...
 ;;; and because READ generally returns anything.
 (defknown (read read-preserving-whitespace)
-  (&optional stream-designator t t t) t (explicit-check))
+  (&optional stream-designator t t t) t ())
 
-(defknown read-char (&optional stream-designator t t t) t (explicit-check)
+(defknown read-char (&optional stream-designator t t t) t ()
   :derive-type (read-elt-type-deriver nil 'character nil))
-(defknown read-char-no-hang (&optional stream-designator t t t) t
-  (explicit-check)
+(defknown read-char-no-hang (&optional stream-designator t t t) t ()
   :derive-type (read-elt-type-deriver nil 'character t))
 
-(defknown read-delimited-list (character &optional stream-designator t) list
-  (explicit-check))
+(defknown read-delimited-list (character &optional stream-designator t) list ())
 ;; FIXME: add a type-deriver => (values (or string eof-value) boolean)
-(defknown read-line (&optional stream-designator t t t) (values t boolean)
-  (explicit-check))
-(defknown unread-char (character &optional stream-designator) t
-  (explicit-check))
+(defknown read-line (&optional stream-designator t t t) (values t boolean) ())
+(defknown unread-char (character &optional stream-designator) t ())
 (defknown peek-char (&optional (or character (member nil t))
                                stream-designator t t t) t
-  (explicit-check)
+  ()
   :derive-type (read-elt-type-deriver t 'character nil))
 
-(defknown listen (&optional stream-designator) boolean (flushable explicit-check))
+(defknown listen (&optional stream-designator) boolean (flushable))
 
-(defknown clear-input (&optional stream-designator) null (explicit-check))
+(defknown clear-input (&optional stream-designator) null ())
 
 (defknown read-from-string
   (string &optional t t
@@ -1208,19 +1189,19 @@
           (:junk-allowed t))
   (values (or integer null ()) index))
 
-(defknown read-byte (stream &optional t t) t (explicit-check)
+(defknown read-byte (stream &optional t t) t ()
   :derive-type (read-elt-type-deriver nil 'integer nil))
 
 (defknown (prin1 print princ) (t &optional stream-designator)
   t
-  (any explicit-check)
+  (any)
   :derive-type #'result-type-first-arg)
 
-(defknown output-object (t stream) null (any explicit-check))
-(defknown %write (t stream-designator) t (any explicit-check))
+(defknown output-object (t stream) null (any))
+(defknown %write (t stream-designator) t (any))
 
 (defknown (pprint) (t &optional stream-designator) (values)
-  (explicit-check))
+  ())
 
 (macrolet
     ((deffrob (name keys returns attributes &rest more)
@@ -1243,47 +1224,44 @@
                (:pprint-dispatch t)
                (:suppress-errors t))
           ,returns ,attributes ,@more)))
-  (deffrob write ((:stream stream-designator)) t (any explicit-check)
+  (deffrob write ((:stream stream-designator)) t (any)
     :derive-type #'result-type-first-arg)
 ;;; xxx-TO-STRING functions are not foldable because they depend on
 ;;; the dynamic environment, the state of the pretty printer dispatch
 ;;; table, and probably other run-time factors.
   (deffrob write-to-string () simple-string
-           (unsafely-flushable explicit-check)))
+           (unsafely-flushable)))
 
 (defknown (prin1-to-string princ-to-string) (t) simple-string (flushable))
 (defknown sb!impl::stringify-object (t) simple-string)
 
-(defknown write-char (character &optional stream-designator) character
-  (explicit-check)
+(defknown write-char (character &optional stream-designator) character ()
   :derive-type #'result-type-first-arg)
 
 (defknown (write-string write-line)
   (string &optional stream-designator &key (:start index) (:end sequence-end))
   string
-  (explicit-check)
+  ()
   :derive-type #'result-type-first-arg)
 
 (defknown (terpri finish-output force-output clear-output)
   (&optional stream-designator) null
-  (explicit-check))
+  ())
 
-(defknown fresh-line (&optional stream-designator) boolean
-  (explicit-check))
+(defknown fresh-line (&optional stream-designator) boolean ())
 
-(defknown write-byte (integer stream) integer
-  (explicit-check)
+(defknown write-byte (integer stream) integer ()
   :derive-type #'result-type-first-arg)
 
 ;;; FIXME: complicated :DESTROYED-CONSTANT-ARGS
 (defknown format ((or (member nil t) stream string)
                   (or string function) &rest t)
   (or string null)
-  (explicit-check))
+  ())
 (defknown sb!format::args-exhausted (string integer) nil)
 
 (defknown (y-or-n-p yes-or-no-p) (&optional string &rest t) boolean
-  (explicit-check))
+  ())
 
 ;;;; from the "File System Interface" chapter:
 
@@ -1425,7 +1403,7 @@
 (defknown warn (t &rest t) null)
 (defknown invoke-debugger (condition) nil)
 (defknown break (&optional format-control &rest t) null)
-(defknown make-condition (type-specifier &rest t) condition (explicit-check))
+(defknown make-condition (type-specifier &rest t) condition ())
 (defknown compute-restarts (&optional (or condition null)) list)
 (defknown find-restart (restart-designator &optional (or condition null))
   (or restart null))
@@ -1515,7 +1493,7 @@
 (defknown (get-internal-run-time get-internal-real-time)
   () internal-time (flushable))
 
-(defknown sleep ((real 0)) null (explicit-check))
+(defknown sleep ((real 0)) null ())
 
 (defknown call-with-timing (callable callable &rest t) *
   (call))
@@ -1567,10 +1545,9 @@
 
 ;;; We can't fold this in general because of SATISFIES. There is a
 ;;; special optimizer anyway.
-(defknown %typep (t (or type-specifier ctype)) boolean
-  (movable flushable explicit-check))
+(defknown %typep (t (or type-specifier ctype)) boolean (movable flushable))
 (defknown %instance-typep (t (or type-specifier ctype)) boolean
-  (movable flushable explicit-check always-translatable))
+  (movable flushable always-translatable))
 ;;; We should never emit a call to %typep-wrapper
 (defknown %typep-wrapper (t t (or type-specifier ctype)) t
   (movable flushable always-translatable))
@@ -1615,30 +1592,26 @@
 (defknown %odd-key-args-error () nil)
 (defknown %unknown-key-arg-error (t) nil)
 (defknown (%ldb %mask-field) (bit-index bit-index integer) unsigned-byte
-  (movable foldable flushable explicit-check))
+  (movable foldable flushable))
 (defknown (%dpb %deposit-field) (integer bit-index bit-index integer) integer
-  (movable foldable flushable explicit-check))
-(defknown %negate (number) number (movable foldable flushable explicit-check))
+  (movable foldable flushable))
+(defknown %negate (number) number (movable foldable flushable))
 (defknown %check-bound (array index fixnum) index
   (movable foldable flushable dx-safe))
 (defknown data-vector-ref (simple-array index) t
-  (foldable unsafely-flushable explicit-check always-translatable))
+  (foldable unsafely-flushable always-translatable))
 (defknown data-vector-ref-with-offset (simple-array fixnum fixnum) t
-  (foldable unsafely-flushable explicit-check always-translatable))
+  (foldable unsafely-flushable always-translatable))
 (defknown data-nil-vector-ref (simple-array index) nil
-  (explicit-check always-translatable))
+  (always-translatable))
 (defknown data-vector-set (array index t) t
-  (explicit-check always-translatable))
+  (always-translatable))
 (defknown data-vector-set-with-offset (array fixnum fixnum t) t
-  (explicit-check always-translatable))
-(defknown hairy-data-vector-ref (array index) t
-  (foldable explicit-check))
-(defknown hairy-data-vector-set (array index t) t (explicit-check))
-(defknown hairy-data-vector-ref/check-bounds (array index) t
-  (foldable explicit-check))
-(defknown hairy-data-vector-set/check-bounds (array index t)
-  t
-  (explicit-check))
+  (always-translatable))
+(defknown hairy-data-vector-ref (array index) t (foldable))
+(defknown hairy-data-vector-set (array index t) t ())
+(defknown hairy-data-vector-ref/check-bounds (array index) t (foldable))
+(defknown hairy-data-vector-set/check-bounds (array index t) t ())
 (defknown %caller-frame () t (flushable))
 (defknown %caller-pc () system-area-pointer (flushable))
 (defknown %with-array-data (array index (or index null))
@@ -1648,8 +1621,7 @@
   (values (simple-array * (*)) index index index)
   (foldable flushable))
 (defknown %set-symbol-package (symbol t) t ())
-(defknown %coerce-callable-to-fun (callable) function
-    (flushable explicit-check))
+(defknown %coerce-callable-to-fun (callable) function (flushable))
 (defknown array-bounding-indices-bad-error (t t t) nil)
 (defknown sequence-bounding-indices-bad-error (t t t) nil)
 (defknown %find-position
@@ -1670,60 +1642,60 @@
 (defknown (%adjoin %adjoin-eq)
     (t list)
     list
-    (explicit-check flushable))
+    (flushable))
 
 (defknown (%member %member-eq
            %assoc %assoc-eq %rassoc %rassoc-eq)
     (t list)
     list
-    (explicit-check foldable flushable))
+    (foldable flushable))
 
 (defknown (%adjoin-key %adjoin-key-eq)
     (t list function)
     list
-    (explicit-check flushable call))
+    (flushable call))
 
 (defknown (%member-key %member-key-eq
            %assoc-key %assoc-key-eq %rassoc-key %rassoc-key-eq)
   (t list function)
   list
-  (explicit-check foldable flushable call))
+  (foldable flushable call))
 
 (defknown (%assoc-if %assoc-if-not %rassoc-if %rassoc-if-not
            %member-if %member-if-not)
   (function list)
   list
-  (explicit-check foldable flushable call))
+  (foldable flushable call))
 
 (defknown (%assoc-if-key %assoc-if-not-key %rassoc-if-key %rassoc-if-not-key
            %member-if-key %member-if-not-key)
   (function list function)
   list
-  (explicit-check foldable flushable call))
+  (foldable flushable call))
 
 (defknown (%adjoin-test %adjoin-test-not)
     (t list function)
     list
-    (explicit-check flushable call))
+    (flushable call))
 
 (defknown (%member-test %member-test-not
            %assoc-test %assoc-test-not
            %rassoc-test %rassoc-test-not)
     (t list function)
     list
-    (explicit-check foldable flushable call))
+    (foldable flushable call))
 
 (defknown (%adjoin-key-test %adjoin-key-test-not)
     (t list function function)
     list
-    (explicit-check flushable call))
+    (flushable call))
 
 (defknown (%member-key-test %member-key-test-not
            %assoc-key-test %assoc-key-test-not
            %rassoc-key-test %rassoc-key-test-not)
     (t list function function)
     list
-    (explicit-check foldable flushable call))
+    (foldable flushable call))
 
 (defknown %check-vector-sequence-bounds (vector index sequence-end)
   index
@@ -1760,8 +1732,7 @@
 (defknown %setnth (unsigned-byte list t) t ()
   :destroyed-constant-args (nth-constant-args 2)
   :derive-type #'result-type-last-arg)
-(defknown %set-fill-pointer (complex-vector index) index
-    (explicit-check)
+(defknown %set-fill-pointer (complex-vector index) index ()
   :destroyed-constant-args (nth-constant-args 1)
   :derive-type #'result-type-last-arg)
 
@@ -1803,7 +1774,7 @@
 (defknown coerce-to-condition ((or condition symbol string function)
                                list type-specifier symbol)
     condition
-    (explicit-check))
+    ())
 
 (defknown coerce-symbol-to-fun (symbol)
   function
@@ -1833,8 +1804,6 @@
 ;;;; atomic ops
 (defknown %compare-and-swap-svref (simple-vector index t t) t
     ())
-(defknown %compare-and-swap-instance-ref (instance index t t) t
-    ())
 (defknown %compare-and-swap-symbol-value (symbol t t) t
     (unwind))
 (defknown (%atomic-dec-symbol-global-value %atomic-inc-symbol-global-value)
@@ -1843,3 +1812,17 @@
     (cons fixnum) fixnum)
 (defknown spin-loop-hint () (values)
     (always-translatable))
+
+;;;; PCL
+
+(defknown sb!pcl::pcl-instance-p (t) boolean
+  (movable foldable flushable))
+
+;; FIXME: should T be be (OR INSTANCE FUNCALLABLE-INSTANCE) etc?
+(defknown slot-value (t symbol) t (any))
+(defknown (slot-boundp slot-exists-p) (t symbol) boolean)
+(defknown sb!pcl::set-slot-value (t symbol t) t (any))
+
+(defknown find-class (symbol &optional t lexenv-designator) (or class null) ())
+(defknown class-of (t) class (flushable))
+(defknown class-name (class) symbol (flushable))
