@@ -275,6 +275,7 @@
 
       #-sb-xc-host ;; Supress &OPTIONAL + &KEY syle-warning on xc host
       (when (and (logtest (bits &key) seen) optional (not silent))
+        ;; FIXME: add a condition class for this
         (style-warn-once
          list "&OPTIONAL and &KEY found in the same lambda list: ~S" list))
 
@@ -1063,8 +1064,7 @@
           ((:environment envp) t) (wrap-block name))
   (declare (type (member t nil :ignore) envp))
   (declare (type (member nil :external :internal) doc-string-allowed))
-  (binding* (((forms decls docstring)
-              (parse-body body :doc-string-allowed doc-string-allowed))
+  (binding* (((forms decls docstring) (parse-body body doc-string-allowed))
              ;; Parse the lambda list, but not recursively.
              ((llks req opt rest keys aux env whole)
               (parse-lambda-list
