@@ -58,7 +58,7 @@
 ;;;
 (defun symeval (symbol env sexpr)
   (binding* (((binding kind frame-ptr expansion) (find-lexical-var env symbol))
-             (type (var-type-assertion env frame-ptr (or binding symbol) :read)))
+             (type (var-type-assertion env symbol binding :read)))
     (if kind ; lexically apparent binding (possibly special or macro)
         (if (eq kind :macro)
             ;; digest-form would lose the type constraint, so pass it in using
@@ -116,7 +116,7 @@
 
 (defun immediate-setq-1 (symbol newval env)
   (binding* (((binding kind frame-ptr value) (find-lexical-var env symbol))
-             (type (var-type-assertion env frame-ptr (or binding symbol) :write)))
+             (type (var-type-assertion env symbol binding :write)))
     (flet ((setf-it (expansion)
              (%eval `(setf ,expansion ,(cast-to type newval)) env))
            (eval-it ()
@@ -154,7 +154,7 @@
 ;;; The globaldb cookie will take care of any loose ends.
 (defun deferred-setq-1 (symbol newval env sexpr)
   (binding* (((binding kind frame-ptr expansion) (find-lexical-var env symbol))
-             (type (var-type-assertion env frame-ptr (or binding symbol) :write)))
+             (type (var-type-assertion env symbol binding :write)))
     (aver (neq type *universal-type*))
     (if kind ; lexically apparent binding (possibly special or macro)
         (if (eq kind :macro)
@@ -284,15 +284,16 @@
        (enforce-single-type (type &rest values)
          (cond ((null values)
                 (error "~S received no values"
-                       (list 'the (type-specifier type))))
+                       (list 'the (specifier-from-checkfun type))))
                ((itypep (first values) type)
                 (apply #'values values))
                (t
-                (error 'type-error :datum (first values)
-                                   :expected-type (type-specifier type)))))
+                (error 'type-error
+                       :datum (first values)
+                       :expected-type (specifier-from-checkfun type)))))
 
        (parse-type (spec-or-obj)
-         (checkable-type
+         (type-checker
           (if (ctype-p spec-or-obj)
               spec-or-obj
               (values-specifier-type spec-or-obj)))))
@@ -513,77 +514,81 @@
 ;;; deeper subforms, but picking off the shallow cases is easy.
 ;;;
 (defun parse-tagbody (body env)
-  (labels ((conditional-go-p (form tags)
-             ;; Given ({WHEN|UNLESS} TEST <forms> ... (GO tag)) rewrite as
-             ;; -> (op (TEST) <forms> T) (GO tag))
-             ;; but if <forms> is empty, only encode the test and the go tag
-             ;; in a way that indicates the test sense.
-             (and (member (car form) '(when unless))
-                  (consp (cddr form))
-                  (let ((tag (go-p (car (last form)) tags)))
-                    (and tag
-                         (let ((op (car form))
-                               (test (cadr form))
-                               (more (butlast (cddr form))))
-                           (declare (index tag))
-                           (if more
-                               (cons (%sexpr `(,op ,test ,@more t)) tag)
-                               (cons (%sexpr test)
-                                     (if (eq op 'when) tag (- tag)))))))))
-           (go-p (form tags)
-             (and (typep form '(cons (eql go) (cons t null)))
-                  (cdr (assoc (cadr form) tags)))))
-  (collect ((tags) (forms))
-    (let ((line-num 1)
-          (optimize-p (and (not (find-lexical-fun env 'when))
-                           (not (find-lexical-fun env 'unless)))))
+  (collect ((tags) (new-body))
+    ;; First pass: just collect the tags
+    (let (any-forms)
       (dolist (item body)
-        (cond ((or (symbolp item) (integerp item))
+        (cond ((consp item) (setq any-forms t))
+              ((or (symbolp item) (integerp item))
                (if (assoc item (tags))
                    (ip-error "Duplicate tag ~S in tagbody" item)
-                   (tags (cons item line-num))))
-              ((atom item)
-               (ip-error "Bad thing to appear in a tagbody: ~S" item))
+                   (tags (list item))))
               (t
-               (forms item)
-               (incf line-num))))
-      (cond ((not (forms))
+               (ip-error "Bad thing to appear in a tagbody: ~S" item))))
+      (cond ((not any-forms)
              (return-from parse-tagbody (return-constant nil)))
             ((not (tags))
-             (return-from parse-tagbody (digest-progn `(,@body nil)))))
-      (when optimize-p
-        ;; Try to convert IF ... GO into WHEN or UNLESS.
-        (let ((forms (forms)) (index 1))
-          (loop
-           (let ((form (car forms)))
-             ;; If FORM is a well-formed IF form and either branch
-             ;; is a GO within this tagbody, rewrite it.
-             (when (typep form
-                          '(cons (eql if)
-                                 (cons t (cons t (or null (cons t null))))))
-               (destructuring-bind (test then &optional else) (cdr form)
-                 (when (cond ((go-p then (tags))
-                              (rplaca forms `(when ,test ,then))
-                              (when else
-                                (rplacd forms (cons else (cdr forms))))
-                              t)
-                             ((go-p else (tags))
-                              (rplaca forms `(unless ,test ,else))
-                              (rplacd forms (cons then (cdr forms)))))
-                   (dolist (tag (tags))
-                     (when (> (cdr tag) index) (incf (cdr tag)))))))
-             (incf index)
-             (pop forms)
-             (unless forms
-               (return))))))
-      (let ((body (coerce (cons (tags) (forms)) 'vector)))
-        (do ((i (1- (length body)) (1- i)))
-            ((zerop i) (handler #'eval-tagbody body))
-          (let ((expr (svref body i)))
-            (setf (svref body i)
-                  (or (go-p expr (tags))
-                      (and optimize-p (conditional-go-p expr (tags)))
-                      (%sexpr expr))))))))))
+             (return-from parse-tagbody (digest-progn `(,@body nil))))))
+    (flet ((go-p (form)
+             (and (typep form '(cons (eql go) (cons t null)))
+                  (assoc (cadr form) (tags)))))
+      ;; Second pass: rewrite conditioned GO forms appearing directly in
+      ;; this tagbody which transfer control to a tag in this tagbody.
+      ;; Local function bindings for WHEN/UNLESS will inhibit this.
+      (dolist (item (unless (or (find-lexical-fun env 'when)
+                                (find-lexical-fun env 'unless))
+                      body))
+        (or (and (typep item '(cons (eql if)
+                                    (cons t (cons t (or null (cons t null))))))
+                 ;; Look for IF forms in which either consequent is a branch.
+                 (destructuring-bind (test then &optional else) (cdr item)
+                   (multiple-value-bind (op branch fallthru)
+                       (cond ((go-p then) (values 'when then else))
+                             ((go-p else) (values 'unless else then)))
+                     (when op
+                       (new-body `(,op ,test ,branch))
+                       (when fallthru
+                         (new-body
+                          (if (atom fallthru) `(progn ,fallthru) fallthru)))
+                       t))))
+            ;; Transform a WHEN or UNLESS if there is more than one subform
+            ;; in the consequent, and the last subform is a control transfer.
+            ;; This is necessary because the tagbody handler recognizes
+            ;; only (WHEN|UNLESS c (GO tag)) as a fast conditional GO.
+            ;; (WHEN (TEST) (stmt1) ... (stmtN) (GO tag)) becomes
+            ;;   (WHEN (WHEN (TEST) (stmt1) ... (stmtN) T) (GO tag))
+            ;; (UNLESS (TEST) (stmt1) ... (stmtN) (GO tag)) becomes
+            ;;   (WHEN (UNLESS (TEST) (stmt1) ... (stmtN) T) (GO tag))
+            (and (typep item '(cons (member when unless)))
+                 (cdddr item)
+                 (awhen (go-p (car (last item)))
+                   (new-body
+                    `(when (,(car item) ,(cadr item) ,@(butlast (cddr item)) t)
+                       (go ,(car it))))))
+            (new-body item))) ; everything else
+      (let ((body (or (new-body) body)))
+        ;; Next assign tags their indices. Interleaving this with the final
+        ;; pass would need a fixup step for forward branches, so do this first.
+        (let ((line-num 1))
+          (dolist (item body)
+            (if (atom item)
+                (rplacd (assoc item (tags)) line-num)
+                (incf line-num))))
+        ;; Collect the executable statements.
+        (let ((line-num 1)
+              (lines (make-array (- (1+ (length body)) (length (tags))))))
+          (setf (aref lines 0) (tags))
+          (dolist (form body (handler #'eval-tagbody lines))
+            (unless (atom form)
+              (setf (aref lines (prog1 line-num (incf line-num)))
+                    (acond ((go-p form) (cdr it)) ; just the line number
+                           ((and (typep form '(cons (member when unless)))
+                                 (singleton-p (cddr form))
+                                 (go-p (third form)))
+                            (let ((line (cdr it)))
+                              (cons (%sexpr (second form))
+                                    (if (eq (car form) 'when) line (- line)))))
+                           (t (%sexpr form)))))))))))
 
 (defun eval-tagbody (code env sexpr)
   (declare (simple-vector code) (ignore sexpr)
@@ -684,14 +689,14 @@
 ;;; of the complicated ...
 
 (defun digest-locally (env body)
-  (multiple-value-bind (forms decls) (iparse-body body)
+  (multiple-value-bind (forms decls) (parse-body body nil t)
     (if (not decls)
         (digest-progn forms)
         (let* ((specials (free-specials env decls))
                (scope (process-typedecls
                        (%make-local-scope decls (new-policy env decls)
                                           (%progn forms) specials)
-                       env specials)))
+                       env 0 specials)))
           (hlambda LOCALLY (scope) (env)
             (enforce-types scope env)
             (dispatch (local-scope-body scope)
@@ -700,40 +705,39 @@
 
 (defspecial locally (&body body)
   :immediate (env)
-  (multiple-value-bind (forms decls) (iparse-body body)
+  (multiple-value-bind (forms decls) (parse-body body nil t)
     (let* ((specials (free-specials env decls))
            (scope (process-typedecls
                    (make-decl-scope decls (new-policy env decls))
-                   env specials)))
+                   env 0 specials)))
       (enforce-types scope env)
       (eval-progn forms (make-basic-env env nil specials scope))))
   :deferred (env) (digest-locally env body))
 
 (defun parse-symbol-macrolet (env bindings body wrap-fn)
-  (multiple-value-bind (forms decls) (iparse-body body)
-    (let* ((specials (declared-specials decls))
-           (n-specials (length specials))
-           (n-macros (length bindings))
-           (symbols (make-array (+ n-specials n-macros)))
-           (expansions (make-array n-macros))
-           (checker (sb-c::symbol-macrolet-definitionize-fun :eval))
-           (index -1))
+  (multiple-value-bind (forms decls) (parse-body body nil t)
+    (binding* (((specials n-specials) (declared-specials decls))
+               (n-macros (length bindings))
+               (symbols (make-array (+ n-macros n-specials)))
+               (expansions (make-array n-macros))
+               (checker (sb-c::symbol-macrolet-definitionize-fun :eval))
+               (index -1))
       (with-package-lock-context (env)
         (dolist (binding bindings)
           (let* ((binding (funcall checker binding))
                  (symbol (car binding))
                  (expansion (cddr binding)))
-            (setf (svref symbols (incf index)) (cons symbol *universal-type*)
+            (setf (svref symbols (incf index)) (list symbol)
                   (svref expansions index) expansion)))
         (assert-declarable-as-special env specials))
       (dolist (symbol specials)
         (if (find symbol symbols :end n-macros :key #'car)
             (ip-error "~S can not be both a symbol-macro and special" symbol))
-        (setf (svref symbols (incf index)) (list symbol)))
+        (setf (svref symbols (incf index)) (find-special-binding env symbol)))
       (process-typedecls
        (%make-symbol-macro-scope decls (new-policy env decls)
                                  symbols expansions (funcall wrap-fn forms))
-       env symbols))))
+       env n-macros symbols))))
 
 (macrolet ((new-env ()
              `(make-symbol-macro-env env
@@ -753,47 +757,42 @@
             (enforce-types scope env)
             (dispatch (symbol-macro-body scope) (new-env)))))))
 
-(defun parse-let (operator env bindings body specials-listifier)
-  (multiple-value-bind (forms decls) (iparse-body body)
-    (let ((declared-specials (declared-specials decls))
-          (n-bound (length bindings))
-          (n-free 0))
+(defun parse-let (maker env bindings body specials-listifier)
+  (multiple-value-bind (forms decls) (parse-body body nil t)
+    (binding* (((declared-specials n-declared) (declared-specials decls))
+               (n-bindings 0)
+               (n-free-specials
+                (let ((boundp 0)) ; mask over declared-specials of bound ones
+                  (dolist (binding bindings (- n-declared (logcount boundp)))
+                    (let ((p (posq (binding-symbol binding) declared-specials)))
+                      (when p (setf (logbitp p boundp) t)))
+                    (incf n-bindings))))
+               (symbols (make-array (+ n-bindings n-free-specials)))
+               (values (make-array n-bindings))
+               (index -1))
+      (dolist (binding bindings)
+        (multiple-value-bind (symbol value-form)
+            (if (atom binding)
+                (values binding nil)
+                (with-subforms (symbol &optional value) binding
+                  (values symbol value)))
+          (unless (symbolp symbol)
+            (ip-error "~S is not a symbol" symbol))
+          (incf index)
+          (setf (svref symbols index) (list symbol)
+                (svref values index) value-form)))
       (dolist (sym declared-specials)
-        (unless (member sym bindings
-                        :key (lambda (x) (if (listp x) (car x) x)))
-          (incf n-free)))
-      (let ((symbols (make-array (+ n-bound n-free)))
-            (values (make-array n-bound))
-            (index -1))
-        (dolist (binding bindings)
-          (multiple-value-bind (symbol value-form)
-              (if (atom binding)
-                  (values binding nil)
-                  (with-subforms (symbol &optional value) binding
-                    (values symbol value)))
-            (unless (symbolp symbol)
-              (ip-error "~S is not a symbol" symbol))
-            (incf index)
-            (setf (svref symbols index) (cons symbol *universal-type*)
-                  (svref values index) value-form)))
-        (dolist (symbol declared-specials)
-          (unless (find (the symbol symbol) symbols :end n-bound :key #'car)
-            (setf (svref symbols (incf index)) (list symbol))))
-        (let ((special-b
-               (mark-bound-specials env declared-specials symbols n-bound)))
-          (process-typedecls
-           (let ((new-policy (new-policy env decls))
-                 (forms (%progn forms))
-                 (specials (funcall specials-listifier
-                                    (collect-progv-symbols symbols n-bound
-                                                           special-b))))
-             (if (eq operator 'let)
-                 (make-let-frame
-                  decls new-policy symbols special-b values forms specials)
-                 (make-let*-frame
-                  decls (env-policy env) new-policy symbols special-b values
-                  forms specials)))
-           env symbols))))))
+        (unless (find (the symbol sym) symbols :end n-bindings :key #'car)
+          (setf (svref symbols (incf index)) (find-special-binding env sym))))
+      (let ((special-b ; mask over symbols of special ones
+             (mark-bound-specials env declared-specials symbols n-bindings)))
+        (process-typedecls
+         (funcall maker decls (new-policy env decls) symbols special-b
+                  values (%progn forms)
+                  (funcall specials-listifier
+                           (collect-progv-symbols symbols n-bindings
+                                                  special-b)))
+         env n-bindings symbols)))))
 
 (defglobal *let-processor* nil)
 
@@ -809,8 +808,8 @@
               `((when (singleton-p bindings)
                   (return-from let*
                    (funcall *let-processor* `(,bindings ,@body) env)))))
-          (let ((frame (parse-let ',operator env bindings body
-                                  ,transform-specials)))
+          (let ((frame (parse-let ',(symbolicate "MAKE-" operator "-FRAME")
+                                  env bindings body ,transform-specials)))
             (let ((values (frame-values frame)))
               (map-into values #'%sexpr values))
             (symbol-macrolet ((symbols (frame-symbols frame)))
@@ -915,11 +914,11 @@
 ;;; MACROLET has an immediate-mode handler since it is common at toplevel.
 (defspecial macrolet (defs &body body)
   :immediate (env)
-  (multiple-value-bind (forms decls) (iparse-body body)
+  (multiple-value-bind (forms decls) (parse-body body nil t)
     (let* ((specials (free-specials env decls))
            (scope (process-typedecls
                    (make-decl-scope decls (new-policy env decls))
-                   env specials)))
+                   env 0 specials)))
       (enforce-types scope env)
       (eval-progn forms
                   (make-macro-env env
@@ -928,7 +927,7 @@
   :deferred (env)
   (if (not defs)
       (digest-locally env body)
-      (multiple-value-bind (forms decls) (iparse-body body)
+      (multiple-value-bind (forms decls) (parse-body body nil t)
         (let ((scope (make-local-fn-scope decls (eval-local-macros env defs)
                                           forms env)))
           (hlambda MACROLET (scope) (env)
@@ -945,7 +944,7 @@
 (defun digest-local-fns (env kind bindings body) ; KIND is FLET or LABELS
   (flet ((proto-functionize (def)
            (with-subforms (name lambda-list &body body) def
-             (multiple-value-bind (forms decls docstring) (iparse-body body t)
+             (multiple-value-bind (forms decls docstring) (parse-body body t t)
                ;; *** Test LEGAL-FUN-NAME-P before asking for an info-value.
                ;; This is because (INFO :FUNCTION :KIND) uses FBOUNDP when it
                ;; does not have an entry for NAME. But calling FBOUNDP
@@ -987,7 +986,7 @@
                                    forms)))
                    (%make-proto-fn `(,kind ,name) lambda-list decls forms
                                    docstring)))))))
-    (multiple-value-bind (forms decls) (iparse-body body)
+    (multiple-value-bind (forms decls) (parse-body body nil t)
       (make-local-fn-scope decls (map 'vector #'proto-functionize bindings)
                            forms env))))
 
@@ -1206,7 +1205,7 @@
                 (mapcar
                  (lambda (f)
                    `(cons ',f
-                          (named-lambda (EVAL ,(symbolicate "2-ARG-" f))
+                          (named-lambda (.eval. ,(symbolicate "2-ARG-" f))
                                         (data env sexpr)
                             (declare (ignore sexpr)
                                      (optimize (sb-c:verify-arg-count 0))
