@@ -24,14 +24,11 @@
 #include "os.h"
 #include "interrupt.h"
 #include "lispregs.h"
-#ifdef LISP_FEATURE_GENCGC
 #include <wchar.h>
 #include "arch.h"
-#include "gencgc-alloc-region.h"
 #include "genesis/compiled-debug-fun.h"
 #include "genesis/compiled-debug-info.h"
 #include "genesis/package.h"
-#endif
 #include "genesis/static-symbols.h"
 #include "genesis/primitive-objects.h"
 #include "thread.h"
@@ -40,6 +37,172 @@
 # include <dlfcn.h>
 #endif
 
+static void
+sbcl_putwc(wchar_t c, FILE *file)
+{
+#ifdef LISP_FEATURE_OS_PROVIDES_PUTWC
+    putwc(c, file);
+#else
+    if (c < 256) {
+        fputc(c, file);
+    } else {
+        fputc('?', file);
+    }
+#endif
+}
+
+struct compiled_debug_fun *
+debug_function_from_pc (struct code* code, void *pc)
+{
+  uword_t code_header_len = sizeof(lispobj) * HeaderValue(code->header);
+  uword_t offset
+    = (uword_t) pc - (uword_t) code - code_header_len;
+  struct compiled_debug_fun *df;
+  struct compiled_debug_info *di;
+  struct vector *v;
+  int i, len;
+
+  if (lowtag_of(code->debug_info) != INSTANCE_POINTER_LOWTAG)
+    return 0;
+
+  di = (struct compiled_debug_info *) native_pointer(code->debug_info);
+  v = (struct vector *) native_pointer(di->fun_map);
+  len = fixnum_value(v->length);
+  df = (struct compiled_debug_fun *) native_pointer(v->data[0]);
+
+  if (len == 1)
+    return df;
+
+  for (i = 1;; i += 2) {
+    unsigned next_pc;
+
+    if (i == len)
+      return ((struct compiled_debug_fun *) native_pointer(v->data[i - 1]));
+
+    if (offset >= (uword_t)fixnum_value(df->elsewhere_pc)) {
+      struct compiled_debug_fun *p
+        = ((struct compiled_debug_fun *) native_pointer(v->data[i + 1]));
+      next_pc = fixnum_value(p->elsewhere_pc);
+    } else
+      next_pc = fixnum_value(v->data[i]);
+
+    if (offset < next_pc)
+      return ((struct compiled_debug_fun *) native_pointer(v->data[i - 1]));
+  }
+
+  return NULL;
+}
+
+static void
+print_string (lispobj *object)
+{
+  int tag = widetag_of(*object);
+  struct vector *vector = (struct vector *) object;
+
+#define doit(TYPE)                              \
+  do {                                          \
+    int i;                                      \
+    int n = fixnum_value(vector->length);       \
+    TYPE *data = (TYPE *) vector->data;         \
+    for (i = 0; i < n; i++) {                   \
+      wchar_t c = (wchar_t) data[i];            \
+      if (c == '\\' || c == '"')                \
+        putchar('\\');                          \
+      sbcl_putwc(c, stdout);                    \
+    }                                           \
+  } while (0)
+
+  switch (tag) {
+  case SIMPLE_BASE_STRING_WIDETAG:
+    doit(unsigned char);
+    break;
+#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
+  case SIMPLE_CHARACTER_STRING_WIDETAG:
+    doit(unsigned int);
+    break;
+#endif
+  default:
+    printf("<??? type %d>", tag);
+  }
+#undef doit
+}
+
+static int string_equal (lispobj *object, char *string)
+{
+    int tag = widetag_of(*object);
+    struct vector *vector = (struct vector *) object;
+
+    if (tag != SIMPLE_BASE_STRING_WIDETAG)
+        return 0;
+    return !strcmp((char *) vector->data, string);
+}
+
+static void
+print_entry_name (lispobj name)
+{
+  if (lowtag_of (name) == LIST_POINTER_LOWTAG) {
+    putchar('(');
+    while (name != NIL) {
+      struct cons *cons = (struct cons *) native_pointer(name);
+      print_entry_name(cons->car);
+      name = cons->cdr;
+      if (name != NIL)
+        putchar(' ');
+    }
+    putchar(')');
+  } else if (lowtag_of(name) == OTHER_POINTER_LOWTAG) {
+    lispobj *object = (lispobj *) native_pointer(name);
+    if (widetag_of(*object) == SYMBOL_HEADER_WIDETAG) {
+      struct symbol *symbol = (struct symbol *) object;
+      if (symbol->package != NIL) {
+        struct package *pkg
+          = (struct package *) native_pointer(symbol->package);
+        lispobj pkg_name = pkg->_name;
+        if (string_equal(native_pointer(pkg_name), "COMMON-LISP"))
+            ;
+        else if (string_equal(native_pointer(pkg_name), "COMMON-LISP-USER")) {
+            fputs("CL-USER::", stdout);
+        }
+        else if (string_equal(native_pointer(pkg_name), "KEYWORD")) {
+            putchar(':');
+        } else {
+            print_string(native_pointer(pkg_name));
+            fputs("::", stdout);
+        }
+      }
+      print_string(native_pointer(symbol->name));
+    } else if (widetag_of(*object) == SIMPLE_BASE_STRING_WIDETAG
+#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
+               || widetag_of(*object) == SIMPLE_CHARACTER_STRING_WIDETAG
+#endif
+               ) {
+      putchar('"');
+      print_string(object);
+      putchar('"');
+    } else {
+      printf("<??? type %d>", (int) widetag_of(*object));
+    }
+  } else {
+    printf("<??? lowtag %d>", (int) lowtag_of(name));
+  }
+}
+
+static void
+print_entry_points (struct code *code)
+{
+    lispobj function = code->entry_points;
+
+    while (function != NIL) {
+        struct simple_fun *header = (struct simple_fun *) native_pointer(function);
+        print_entry_name(header->name);
+
+        function = header->next;
+        if (function != NIL)
+            printf (", ");
+    }
+}
+
+
 #if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
 
 /* KLUDGE: Sigh ... I know what the call frame looks like and it had
@@ -169,6 +332,7 @@ previous_info(struct call_info *info)
     struct call_frame *this_frame;
     struct thread *thread=arch_os_get_current_thread();
     int free_ici;
+    lispobj lra;
 
     if (!cs_valid_pointer_p(info->frame)) {
         printf("Bogus callee value (0x%08lx).\n", (uword_t)info->frame);
@@ -182,30 +346,34 @@ previous_info(struct call_info *info)
 
     if (info->frame == NULL || info->frame == this_frame)
         return 0;
-
-    if (info->lra == NIL) {
+    lra = info->lra;
+    if (lra == NIL) {
         /* We were interrupted. Find the correct signal context. */
         free_ici = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,thread));
         while (free_ici-- > 0) {
             os_context_t *context =
                 thread->interrupt_contexts[free_ici];
             if ((struct call_frame *)(uword_t)
-                    (*os_context_register_addr(context, reg_CFP))
+                (*os_context_register_addr(context, reg_CFP))
                 == info->frame) {
                 call_info_from_context(info, context);
                 break;
             }
         }
-    }
-    else {
-        info->code = code_pointer(info->lra);
+    } else if (fixnump(lra)) {
+        info->code = native_pointer(this_frame->code);
+        info->pc = (uword_t)(info->code + lra);
+        info->lra = NIL;
+    } else {
+        info->code = code_pointer(lra);
+
         if (info->code != NULL)
             info->pc = (uword_t)native_pointer(info->lra) -
                 (uword_t)info->code -
 #ifndef LISP_FEATURE_ALPHA
                 (HEADER_LENGTH(info->code->header) * sizeof(lispobj));
 #else
-                (HEADER_LENGTH(((struct code *)info->code)->header) * sizeof(lispobj));
+        (HEADER_LENGTH(((struct code *)info->code)->header) * sizeof(lispobj));
 #endif
         else
             info->pc = 0;
@@ -218,70 +386,37 @@ void
 lisp_backtrace(int nframes)
 {
     struct call_info info;
-
+    int i = 0;
     call_info_from_lisp_state(&info);
 
     do {
-        printf("<Frame 0x%08lx%s, ", (uword_t) info.frame,
-                info.interrupted ? " [interrupted]" : "");
+        printf("%4d: ", i);
 
         if (info.code != (struct code *) 0) {
-            lispobj function;
+            struct compiled_debug_fun *df ;
+            if (info.lra != NIL &&
+                (df = debug_function_from_pc((struct code *)info.code, (void *)info.lra)))
+                print_entry_name(df->name);
+            else
+                print_entry_points((struct code *)info.code);
 
-            printf("CODE: 0x%08lX, ", (uword_t) info.code | OTHER_POINTER_LOWTAG);
-
-#ifndef LISP_FEATURE_ALPHA
-            function = info.code->entry_points;
-#else
-            function = ((struct code *)info.code)->entry_points;
-#endif
-            while (function != NIL) {
-                struct simple_fun *header;
-                lispobj name;
-
-                header = (struct simple_fun *) native_pointer(function);
-                name = header->name;
-
-                if (lowtag_of(name) == OTHER_POINTER_LOWTAG) {
-                    lispobj *object;
-
-                    object = (lispobj *) native_pointer(name);
-
-                    if (widetag_of(*object) == SYMBOL_HEADER_WIDETAG) {
-                        struct symbol *symbol;
-
-                        symbol = (struct symbol *) object;
-                        object = (lispobj *) native_pointer(symbol->name);
-                    }
-                    if (widetag_of(*object) == SIMPLE_BASE_STRING_WIDETAG) {
-                        struct vector *string;
-
-                        string = (struct vector *) object;
-                        printf("%s, ", (char *) string->data);
-                    } else
-                        /* FIXME: broken from (VECTOR NIL) */
-                        printf("(Not simple string??\?), ");
-                } else
-                    printf("(Not other pointer??\?), ");
-
-
-                function = header->next;
-            }
+            printf(" %p", (uword_t) info.code | OTHER_POINTER_LOWTAG);
         }
         else
-            printf("CODE: ???, ");
+            printf("CODE = ???");
+        printf("%s fp = %p", info.interrupted ? " [interrupted]" : "",
+               info.frame);
 
         if (info.lra != NIL)
-            printf("LRA: 0x%08lx, ", (uword_t)info.lra);
+            printf(" LRA = %p", info.lra);
         else
-            printf("<no LRA>, ");
+            printf(" <no LRA>");
 
         if (info.pc)
-            printf("PC: 0x%x>\n", info.pc);
-        else
-            printf("PC: ??\?>\n");
+            printf(" pc = %p", info.pc);
+        putchar('\n');
 
-    } while (--nframes > 0 && previous_info(&info));
+    } while (i++ < nframes && previous_info(&info));
 }
 
 #else
@@ -347,152 +482,6 @@ x86_call_context (void *fp, void **ra, void **ocfp)
   return 1;
 }
 
-struct compiled_debug_fun *
-debug_function_from_pc (struct code* code, void *pc)
-{
-  uword_t code_header_len = sizeof(lispobj) * HeaderValue(code->header);
-  uword_t offset
-    = (uword_t) pc - (uword_t) code - code_header_len;
-  struct compiled_debug_fun *df;
-  struct compiled_debug_info *di;
-  struct vector *v;
-  int i, len;
-
-  if (lowtag_of(code->debug_info) != INSTANCE_POINTER_LOWTAG)
-    return 0;
-
-  di = (struct compiled_debug_info *) native_pointer(code->debug_info);
-  v = (struct vector *) native_pointer(di->fun_map);
-  len = fixnum_value(v->length);
-  df = (struct compiled_debug_fun *) native_pointer(v->data[0]);
-
-  if (len == 1)
-    return df;
-
-  for (i = 1;; i += 2) {
-    unsigned next_pc;
-
-    if (i == len)
-      return ((struct compiled_debug_fun *) native_pointer(v->data[i - 1]));
-
-    if (offset >= (uword_t)fixnum_value(df->elsewhere_pc)) {
-      struct compiled_debug_fun *p
-        = ((struct compiled_debug_fun *) native_pointer(v->data[i + 1]));
-      next_pc = fixnum_value(p->elsewhere_pc);
-    } else
-      next_pc = fixnum_value(v->data[i]);
-
-    if (offset < next_pc)
-      return ((struct compiled_debug_fun *) native_pointer(v->data[i - 1]));
-  }
-
-  return NULL;
-}
-
-static void
-sbcl_putwc(wchar_t c, FILE *file)
-{
-#ifdef LISP_FEATURE_OS_PROVIDES_PUTWC
-    putwc(c, file);
-#else
-    if (c < 256) {
-        fputc(c, file);
-    } else {
-        fputc('?', file);
-    }
-#endif
-}
-
-static void
-print_string (lispobj *object)
-{
-  int tag = widetag_of(*object);
-  struct vector *vector = (struct vector *) object;
-
-#define doit(TYPE)                              \
-  do {                                          \
-    int i;                                      \
-    int n = fixnum_value(vector->length);       \
-    TYPE *data = (TYPE *) vector->data;         \
-    for (i = 0; i < n; i++) {                   \
-      wchar_t c = (wchar_t) data[i];            \
-      if (c == '\\' || c == '"')                \
-        putchar('\\');                          \
-      sbcl_putwc(c, stdout);                    \
-    }                                           \
-  } while (0)
-
-  switch (tag) {
-  case SIMPLE_BASE_STRING_WIDETAG:
-    doit(unsigned char);
-    break;
-#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
-  case SIMPLE_CHARACTER_STRING_WIDETAG:
-    doit(unsigned int);
-    break;
-#endif
-  default:
-    printf("<??? type %d>", tag);
-  }
-#undef doit
-}
-
-static void
-print_entry_name (lispobj name)
-{
-  if (lowtag_of (name) == LIST_POINTER_LOWTAG) {
-    putchar('(');
-    while (name != NIL) {
-      struct cons *cons = (struct cons *) native_pointer(name);
-      print_entry_name(cons->car);
-      name = cons->cdr;
-      if (name != NIL)
-        putchar(' ');
-    }
-    putchar(')');
-  } else if (lowtag_of(name) == OTHER_POINTER_LOWTAG) {
-    lispobj *object = (lispobj *) native_pointer(name);
-    if (widetag_of(*object) == SYMBOL_HEADER_WIDETAG) {
-      struct symbol *symbol = (struct symbol *) object;
-      if (symbol->package != NIL) {
-        struct package *pkg
-          = (struct package *) native_pointer(symbol->package);
-        lispobj pkg_name = pkg->_name;
-        print_string(native_pointer(pkg_name));
-        fputs("::", stdout);
-      }
-      print_string(native_pointer(symbol->name));
-    } else if (widetag_of(*object) == SIMPLE_BASE_STRING_WIDETAG
-#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
-               || widetag_of(*object) == SIMPLE_CHARACTER_STRING_WIDETAG
-#endif
-               ) {
-      putchar('"');
-      print_string(object);
-      putchar('"');
-    } else {
-      printf("<??? type %d>", (int) widetag_of(*object));
-    }
-  } else {
-    printf("<??? lowtag %d>", (int) lowtag_of(name));
-  }
-}
-
-static void
-print_entry_points (struct code *code)
-{
-  lispobj function = code->entry_points;
-
-  while (function != NIL) {
-    struct simple_fun *header = (struct simple_fun *) native_pointer(function);
-    print_entry_name(header->name);
-
-    function = header->next;
-    if (function != NIL)
-      printf (", ");
-  }
-}
-
 void
 describe_thread_state(void)
 {
@@ -522,67 +511,81 @@ describe_thread_state(void)
     printf("Pending handler = %p\n", data->pending_handler);
 }
 
+void print_backtrace_frame(void *pc, void *fp, int i) {
+    lispobj *p;
+    printf("%4d: ", i);
+
+    p = (lispobj *) component_ptr_from_pc((lispobj *) pc);
+
+    if (p) {
+        struct code *cp = (struct code *) p;
+        struct compiled_debug_fun *df = debug_function_from_pc(cp, pc);
+        if (df)
+            print_entry_name(df->name);
+        else
+            print_entry_points(cp);
+        printf(", pc = %p, fp = %p", pc, fp);
+    } else {
+#ifdef LISP_FEATURE_OS_PROVIDES_DLADDR
+        Dl_info info;
+        if (dladdr(pc, &info)) {
+            printf("Foreign function %s, pc = %p, fp = %p", info.dli_sname, pc, fp);
+        } else
+#endif
+            printf("Foreign function, pc = %p, fp = %p", pc, fp);
+    }
+
+    putchar('\n');
+}
+
 /* This function has been split from lisp_backtrace() to enable Lisp
  * backtraces from gdb with call backtrace_from_fp(...). Useful for
  * example when debugging threading deadlocks.
  */
 void
-backtrace_from_fp(void *fp, int nframes)
+backtrace_from_fp(void *fp, int nframes, int start)
 {
-  int i;
+  int i = start;
 
-  for (i = 0; i < nframes; ++i) {
-    lispobj *p;
+  for (; i < nframes; ++i) {
     void *ra;
     void *next_fp;
 
     if (!x86_call_context(fp, &ra, &next_fp))
       break;
-
-    printf("%4d: ", i);
-
-    p = (lispobj *) component_ptr_from_pc((lispobj *) ra);
-    if (p) {
-      struct code *cp = (struct code *) p;
-      struct compiled_debug_fun *df = debug_function_from_pc(cp, ra);
-      if (df)
-        print_entry_name(df->name);
-      else
-        print_entry_points(cp);
-    } else {
-#ifdef LISP_FEATURE_OS_PROVIDES_DLADDR
-        Dl_info info;
-        if (dladdr(ra, &info)) {
-            printf("Foreign function %s, fp = 0x%lx, ra = 0x%lx",
-                   info.dli_sname,
-                   (uword_t) next_fp,
-                   (uword_t) ra);
-        } else
-#endif
-        printf("Foreign fp = %p, ra = %p",
-               (void*) next_fp,
-               (void*) ra);
-    }
-
-    putchar('\n');
+    print_backtrace_frame(ra, next_fp, i);
     fp = next_fp;
   }
 }
 
+void backtrace_from_context(os_context_t *context, int nframes) {
+#ifdef LISP_FEATURE_X86
+    void *fp = (void *)*os_context_register_addr(context,reg_EBP);
+#elif defined (LISP_FEATURE_X86_64)
+    void *fp = (void *)*os_context_register_addr(context,reg_RBP);
+#endif
+    print_backtrace_frame((void *)*os_context_pc_addr(context), fp, 0);
+    backtrace_from_fp(fp, nframes - 1, 1);
+}
+
 void
 lisp_backtrace(int nframes)
 {
-  void *fp;
+    struct thread *thread=arch_os_get_current_thread();
+    int free_ici = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
-#if defined(LISP_FEATURE_X86)
-  asm("movl %%ebp,%0" : "=g" (fp));
+    if (free_ici) {
+        os_context_t *context = thread->interrupt_contexts[free_ici - 1];
+        backtrace_from_context(context, nframes);
+    } else {
+        void *fp;
+
+#ifdef LISP_FEATURE_X86
+        asm("movl %%ebp,%0" : "=g" (fp));
 #elif defined (LISP_FEATURE_X86_64)
-  asm("movq %%rbp,%0" : "=g" (fp));
-#else
-#error "How did we get here?"
+        asm("movq %%rbp,%0" : "=g" (fp));
 #endif
-
-  backtrace_from_fp(fp, nframes);
+        backtrace_from_fp(fp, nframes, 0);
+    }
 }
-
 #endif
